unit Win95File;

{ Demonstration for how to work with long file names
  in Windows 95. This does not work with Windows NT.

  To learn more about the VxD interface in Windows 95,
  read Secrets of Windows 95 Programming, by Matt Pietrek.

  To learn more about long file names in Windows 95,
  read Programmer's Guide to Windows 95.

  Copyright  1996 Tempest Software.

  This software is offered for free, but without support.
  Use it at your own risk.
}

interface

{ Return a complete path name, using the long file name
  for all directories and the file name, if possible.
  If the SrcPath is not a complete path, then use the
  current directory. Interpret all '.' and '..' directories.
  Return the resulting path, or an empty string for an error.
  If an empty string is returned, GetLastError returns
  the DOS error code. }
function GetLongName(FileName: string): string;

{ Get the short file name for a long name.
  Do not expand the path. Return an empty string for an error. }
function GetShortName(FileName: string): string;

{ The GetLongName routine uses the Windows 95 VxD
  implementation of Int 21H. All the VxD calls go through
  a single routine, but with a variable number of arguments
  on the stack. Call the appropriate VxDCall routine,
  according to the number of arguments you have. }
function VxDCall0(ID: Integer): Integer; stdcall;
function VxDCall1(ID, arg1: Integer): Integer; stdcall;
function VxDCall2(ID, arg1, arg2: Integer): Integer; stdcall;
function VxDCall3(ID, arg1, arg2, arg3: Integer): Integer; stdcall;
function VxDCall4(ID, arg1, arg2, arg3, arg4: Integer): Integer; stdcall;
function VxDCall5(ID, arg1, arg2, arg3, arg4, arg5: Integer): Integer; stdcall;
function VxDCall6(ID, arg1, arg2, arg3, arg4, arg5, arg6: Integer): Integer; stdcall;
function VxDCall7(ID, arg1, arg2, arg3, arg4, arg5, arg6, arg7: Integer): Integer; stdcall;
function VxDCall8(ID, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8: Integer): Integer; stdcall;

{ To call an Int 21H DOS call from Windows 95, set up
  the registers and then call Int21Dispatch. }
function Int21Dispatch: Integer;

implementation

uses Windows, SysUtils;

{ Actually, all the VxDCall routines point to the same entry point
  in Kernel32.dll. Different names allow the caller to pass
  a variable number of arguments. }
function VxDCall0; stdcall; external 'kernel32.dll' index 1;
function VxDCall1; stdcall; external 'kernel32.dll' index 2;
function VxDCall2; stdcall; external 'kernel32.dll' index 3;
function VxDCall3; stdcall; external 'kernel32.dll' index 4;
function VxDCall4; stdcall; external 'kernel32.dll' index 5;
function VxDCall5; stdcall; external 'kernel32.dll' index 6;
function VxDCall6; stdcall; external 'kernel32.dll' index 7;
function VxDCall7; stdcall; external 'kernel32.dll' index 8;
function VxDCall8; stdcall; external 'kernel32.dll' index 9;

{ Use the Windows 95 Int 21H interface by calling to
  the VWIN32 VxD. The caller must have set up all the
  registers before calling Int21Dispatch. The carry bit
  is not preserved by calling VxDCall, so you need to
  interpret the returned register values to detect an error. }
function Int21Dispatch: Integer;
asm
  push ecx
  push eax
  push $002A0010
  call VxDCall2
end;

{ Get a long file name. The Dst must have enough room
  to hold the result. Return 0 for success, or a DOS error
  code for failure. }
function GetLongPathName(Src: PChar; Dst: PChar): Integer;
asm
  push esi
  push edi
  mov esi, Src
  mov edi, Dst
  mov eax, $7160
  mov ecx, 2
  call Int21Dispatch
  cmp eax, $4000
  jl @Error
  xor eax, eax
@Error:
  pop edi
  pop esi
end;

{ Return a complete path name, using the long file name
  for all directories and the file name, if possible.
  Return the resulting path, or an empty string for an error.
  If there is an error, set the DOS error code as the
  GetLastError code. }
function GetLongName(FileName: string): string;
var
  Error: Integer;
begin
  SetLength(Result, 260);
  Error := GetLongPathName(PChar(FileName), PChar(Result));
  if Error = 0 then
    SetLength(Result, StrLen(PChar(Result)))
  else
  begin
    SetLastError(Error);
    Result := '';
  end;
end;

{ Get the short name for a long name. Fortunately,
  Microsoft already has a function to do this. Return
  an empty string for an error. }
function GetShortName(FileName: string): string;
begin
  SetLength(Result, 260);
  SetLength(Result, GetShortPathName(PChar(Filename), PChar(Result), Length(Result)));
end;

end.
