unit WinStuff;

interface

uses Windows;

{ These functions were writting using information found
  in Matt Pietrek's Windows 95 System Programming Secrets,
  (1995, IDG Books).

  The code below was written from scratch by Tempest Software
  and is copyrighted by Tempest Software. You are granted
  permission to use this software in a component, library,
  or application, without royalty.

  Copyright  1996 Tempest Software. All rights reserved.
}

{ Convert a ThreadID or ProcessID into a pointer to the
  thread or process DB record. }
function Unobfuscate(ID: DWord): DWord;
{ Return whether a window is Win32 (True) or Win16 (False). }
function IsWin32Window(Handle: HWnd): Boolean;
{ Return whether a thread if Win32 (True) or Win16 (False). }
function IsWin32Thread(Handle: DWord): Boolean;
{ Return whether a process if Win32 (True) or Win16 (False). }
function IsWin32Process(Handle: DWord): Boolean;
{ Return whether a process is running as a console app. }
function IsConsoleProcess(Handle: DWord): Boolean;

implementation

type
  PThreadDB = ^TThreadDB;
  TThreadDB = record
    dwType: DWord;
    cReference: DWord;
    pProcess: Pointer;
    pSomeEvent: DWord;
    pvExcept: DWord;
    TopOfStack: DWord;
    StackLow: DWord;
    W16TDB: Word;
    StackSelector16: Word;
    SelmanList: DWord;
    UserPointer: DWord;
    pTIB: Pointer;
    TIBFlags: DWord;
    Win16MutexCount: Word;
    { lots of other stuff that is not germane to this task... }
  end;
  PProcessDB = ^TProcessDB;
  TProcessDB = record
    dwType: DWord;
    cReference: DWord;
    un1: DWord;
    pSomeEvent: DWord;
    TerminationStatus: DWord;
    un2: DWord;
    DefaultHeap: DWord;
    MemoryContext: DWord;
    Flags: DWord;
    pPSP: DWord;
    { etc. }
  end;

const
  { The only TIBFlags flag that is interesting. }
  TIBF_Win32 = 1;
  { Several of the more interesting process flags. }
  fWin16Process   = $008;
  fDosProcess     = $010;
  fConsoleProcess = $020;
  fServiceProcess = $100;
  { Type values }
  Type_Process = 5;
  Type_Thread = 6;

{ Microsoft XORs the Obfuscator mask with a ThreadDB pointer
  to make a ThreadID. Fortunately, one can learn the Obfuscator
  value and XOR it with the ThreadID to obtain the ThreadDB
  pointer. See Matt Pietrek's Wndows 95 System Programming
  Secrets for more information. }
var
  Obfuscator: DWord = 0;

function Unobfuscate(ID: DWord): DWord;
begin
  if Obfuscator = 0 then
  begin
    Obfuscator := GetCurrentThreadID;
    asm
      mov edx, fs:[18h]     { Get the current TIB pointer }
      sub edx, 10h          { Get the current Thread DB ptr. }
      xor Obfuscator, edx   { XOR with thread ID to learn Obfuscator mask }
    end;
  end;
  Result := ID xor Obfuscator;
end;

function IsWin32Window(Handle: HWnd): Boolean;
var
  ThreadID: DWord;
begin
  ThreadID := GetWindowThreadProcessID(Handle, nil);
  Result := IsWin32Thread(ThreadID);
end;

function IsWin32Thread(Handle: DWord): Boolean;
var
  TDB: PThreadDB;
begin
  TDB := PThreadDB(Unobfuscate(Handle));
  if TDB^.dwType <> Type_Thread then
    Result := False
  else
    Result := 0 <> TIBF_Win32 and TDB^.TIBFlags;
end;

function IsWin32Process(Handle: DWord): Boolean;
var
  PDB: PProcessDB;
begin
  PDB := PProcessDB(Unobfuscate(Handle));
  if PDB^.dwType <> Type_Process then
    Result := False
  else
    Result := 0 = fWin16Process and PDB^.Flags;
end;

function IsConsoleProcess(Handle: DWord): Boolean;
var
  PDB: PProcessDB;
begin
  PDB := PProcessDB(Unobfuscate(Handle));
  if PDB^.dwType <> Type_Process then
    Result := False
  else
    Result := 0 <> fConsoleProcess and PDB^.Flags;
end;

end.
